#include "StdAfx.h"
#include ".\dibsection.h"

CDIBSection::CDIBSection(void)
{
	InitObject();
}

CDIBSection::~CDIBSection(void)
{
	Close();
}

void CDIBSection::InitObject(void)
{
	m_hbmp = NULL ;
	m_pBits = NULL ;
	m_pdc = NULL ;
	m_total_width = 0;
	m_size.cx = m_size.cy = m_total_width = 0;
}

void CDIBSection::Close(void)
{
	try
	{
		if (m_hbmOld)
		{
			::SelectObject(m_pdc->GetSafeHdc(), m_hbmOld);
		}

		if (m_hbmp) DeleteObject(m_hbmp);

		if (m_pdc) delete m_pdc;

		InitObject();
	}
	catch(...)
	{
		throw CString("An unspecified error occurred");
	}
}

void CDIBSection::Create(int cx, int cy, int nbits) 
{
	try
	{
		ASSERT(cx > 0);
		ASSERT(cy > 0);

		ASSERT((nbits == 8) || (nbits == 16) ||
		(nbits == 24) || (nbits == 32))	;

		Close();

		m_size.cx = cx ;
		m_size.cy = cy ;

		int size = sizeof(BITMAPINFOHEADER) ;
		memset(&m_bih, 0, size);

		m_bih.biSize = size;
		m_bih.biWidth = ((((int) cx * 8) + 31) & ~31) >> 3;
		m_bih.biHeight = cy * -1;
		m_bih.biPlanes = 1;
		m_bih.biBitCount = nbits;
		m_bih.biCompression = BI_RGB;
		m_total_width = m_bih.biWidth;

		m_pdc = new CDC ;
		m_pdc->CreateCompatibleDC(NULL);
		m_hbmp = CreateDIBSection( m_pdc->GetSafeHdc(),
								(BITMAPINFO*)&m_bih,
								DIB_PAL_COLORS,
								&m_pBits,
								NULL,
								0);

		ASSERT(m_hbmp);
		ASSERT(m_pBits);

		if (m_hbmp)
		{
			m_hbmOld = (HBITMAP)::SelectObject(m_pdc->GetSafeHdc(), m_hbmp);
		} 
	}
	catch(...)
	{
		throw CString("An unspecified error occurred");
	}
}

void CDIBSection::Draw(CDC* pdcDest, int x, int y) 
{
	try
	{
		pdcDest->StretchBlt(0, 0, m_size.cx, m_size.cy, m_pdc, 0, 0, m_size.cx, m_size.cy, SRCCOPY);
	}
	catch(...)
	{
		throw CString("An unspecified error occurred");
	}
}

void CDIBSection::SetPalette(CPalette* pPal)
{
	try
	{
		ASSERT(pPal);

		int iColors = 0;
		pPal->GetObject(sizeof(iColors), &iColors);
		ASSERT(iColors > 0);
		PALETTEENTRY* pPE = new PALETTEENTRY[iColors];
		pPal->GetPaletteEntries(0, iColors, pPE);

		RGBQUAD* pRGB = new RGBQUAD[iColors];
		ASSERT(pRGB);
		for (int i = 0; i < iColors; i++) 
		{
			pRGB[i].rgbRed = pPE[i].peRed;
			pRGB[i].rgbGreen = pPE[i].peGreen;
			pRGB[i].rgbBlue = pPE[i].peBlue;
			pRGB[i].rgbReserved = 0;
		}

		::SetDIBColorTable(m_pdc->GetSafeHdc(),
						0, iColors,
						pRGB);

		delete [] pRGB;
		delete [] pPE;
	}
	catch(...)
	{
		throw CString("An unspecified error occurred");
	}
}

int CDIBSection::CreatePalette()
{
	if(m_Palette.m_hObject==0){	
		LOGPALETTE LogPal;
		LogPal.palVersion=0x300;
		LogPal.palNumEntries=1;
		m_Palette.CreatePalette(&LogPal);
		BOOL b = m_Palette.ResizePalette(256);
		if(!b) return -1;
	}

	PALETTEENTRY PalEnt[256];
	for(int i=0;i<256;i++){
		PalEnt[i].peRed  =255;//((i>>5)&7)*255/7;
		PalEnt[i].peGreen=0;//((i>>2)&7)*255/7;
		PalEnt[i].peBlue =255;//((i>>0)&3)*255/3;
		PalEnt[i].peFlags=0;
	}
	UINT u=m_Palette.SetPaletteEntries(0,256,PalEnt);

	return u;
}


